#!/usr/bin/env python

'''
TEL-3214 Computer Communication Networks client (ccnc)
Diarmuid O'Briain
'''

# Import libraries

import sys
import socket
import re

# Initialise data structures

BUFFER_SIZE = 1024

# Gather arguments

try: 
    ip_addr = sys.argv[1]
    tcp_port = sys.argv[2]
except:
    print "\nIncorrect format"
    print sys.argv[0], "<ip addr> <port 5001-9999 | 5010>"
    sys.exit()


print '''
Welcome to the TEL-3214 Computer Communication Networks client (ccnc)
This program is designed to give students an understanding of the 
client/server architecture through a simple TCP Socket.
'''

# IP address test functions

def is_valid_ipv4_address(ip_addr):

    # Try convert IPv4 address to binary format. 
    try:
        socket.inet_pton(socket.AF_INET, ip_addr)

    # Failed so it is not a valid IPv4 address. 
    except socket.error:   
        return False

    return True

def is_valid_ipv6_address(ip_addr):

    # Try convert IPv6 address to binary format.
    try:
        socket.inet_pton(socket.AF_INET6, ip_addr)

    # Failed so it is not a valid IPv4 address. 
    except socket.error:   
        return False

    return True
   
# Test the IP address for validity

if (is_valid_ipv4_address(ip_addr) == False and is_valid_ipv6_address(ip_addr) == False):
    print "\nIP address is not valid use the following format"
    print sys.argv[0], "<ip addr> <port 5001-9999 | 5010>"
    sys.exit()
    
# Test the Port number validity

tcp_port = tcp_port.strip()

try: 
    tcp_port = int(tcp_port)
except:
    print "\nPort number is not valid use the following format"
    print sys.argv[0], "<ip addr> <port 5001-9999 | 5010>"
    sys.exit()

if tcp_port < 5001 or tcp_port > 9999:
    print "\nPort number is not valid use the following format"
    print sys.argv[0], "<ip addr> <port 5001-9999 | 5010>"
    sys.exit()    

print "IP Addr: ", ip_addr, "Port#", tcp_port, "\n"

# Get message from user

message = raw_input('What message do you want to pass?: ')

# Socket connect 

s = None
for sai in socket.getaddrinfo(ip_addr, tcp_port, socket.AF_UNSPEC, socket.SOCK_STREAM, 0, socket.AI_PASSIVE):
    addr_family, sock_type, proto, canonical_name, sock_addr = sai
    try:
        s = socket.socket(addr_family, sock_type, proto)
    except socket.error as msg:
        s = None
        continue
    try:
        s.connect(sock_addr)
    except socket.error as msg:
        s.close()
        s = None
        continue
    break
if s is None:
    print 'could not open socket'
    sys.exit(1)

# Sending message

print "\nSending message to server at", ip_addr, "on port number:", tcp_port 
s.sendall(message)

# Reply from server received

data = s.recv(1024)
s.close()

# Print string returned from server

print '\nReceived back: ', data

